import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import dayjs from 'dayjs/esm';

import { DATE_TIME_FORMAT } from 'app/config/input.constants';
import { IReviewHistoryEntry, ReviewHistoryEntry } from '../review-history-entry.model';

import { ReviewHistoryEntryService } from './review-history-entry.service';
import { ReviewStatus } from 'app/entities/enumerations/review-status.model';

describe('ReviewHistoryEntry Service', () => {
  let service: ReviewHistoryEntryService;
  let httpMock: HttpTestingController;
  let elemDefault: IReviewHistoryEntry;
  let expectedResult: IReviewHistoryEntry | IReviewHistoryEntry[] | boolean | null;
  let currentDate: dayjs.Dayjs;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(ReviewHistoryEntryService);
    httpMock = TestBed.inject(HttpTestingController);
    currentDate = dayjs();

    elemDefault = {
      id: 0,
      message: 'AAAAAAA',
      type: ReviewStatus.REVIEW_REQUESTED,
      entryDate: currentDate,
    };
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = Object.assign(
        {
          entryDate: currentDate.format(DATE_TIME_FORMAT),
        },
        elemDefault
      );

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(elemDefault);
    });

    it('should create a ReviewHistoryEntry', () => {
      const returnedFromService = Object.assign(
        {
          id: 0,
          entryDate: currentDate.format(DATE_TIME_FORMAT),
        },
        elemDefault
      );

      const expected = Object.assign(
        {
          entryDate: currentDate,
        },
        returnedFromService
      );

      service.create(new ReviewHistoryEntry()).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a ReviewHistoryEntry', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          message: 'BBBBBB',
          type: 'BBBBBB',
          entryDate: currentDate.format(DATE_TIME_FORMAT),
        },
        elemDefault
      );

      const expected = Object.assign(
        {
          entryDate: currentDate,
        },
        returnedFromService
      );

      service.update(expected).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a ReviewHistoryEntry', () => {
      const patchObject = Object.assign(
        {
          message: 'BBBBBB',
        },
        new ReviewHistoryEntry()
      );

      const returnedFromService = Object.assign(patchObject, elemDefault);

      const expected = Object.assign(
        {
          entryDate: currentDate,
        },
        returnedFromService
      );

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of ReviewHistoryEntry', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          message: 'BBBBBB',
          type: 'BBBBBB',
          entryDate: currentDate.format(DATE_TIME_FORMAT),
        },
        elemDefault
      );

      const expected = Object.assign(
        {
          entryDate: currentDate,
        },
        returnedFromService
      );

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toContainEqual(expected);
    });

    it('should delete a ReviewHistoryEntry', () => {
      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult);
    });

    describe('addReviewHistoryEntryToCollectionIfMissing', () => {
      it('should add a ReviewHistoryEntry to an empty array', () => {
        const reviewHistoryEntry: IReviewHistoryEntry = { id: 123 };
        expectedResult = service.addReviewHistoryEntryToCollectionIfMissing([], reviewHistoryEntry);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(reviewHistoryEntry);
      });

      it('should not add a ReviewHistoryEntry to an array that contains it', () => {
        const reviewHistoryEntry: IReviewHistoryEntry = { id: 123 };
        const reviewHistoryEntryCollection: IReviewHistoryEntry[] = [
          {
            ...reviewHistoryEntry,
          },
          { id: 456 },
        ];
        expectedResult = service.addReviewHistoryEntryToCollectionIfMissing(reviewHistoryEntryCollection, reviewHistoryEntry);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a ReviewHistoryEntry to an array that doesn't contain it", () => {
        const reviewHistoryEntry: IReviewHistoryEntry = { id: 123 };
        const reviewHistoryEntryCollection: IReviewHistoryEntry[] = [{ id: 456 }];
        expectedResult = service.addReviewHistoryEntryToCollectionIfMissing(reviewHistoryEntryCollection, reviewHistoryEntry);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(reviewHistoryEntry);
      });

      it('should add only unique ReviewHistoryEntry to an array', () => {
        const reviewHistoryEntryArray: IReviewHistoryEntry[] = [{ id: 123 }, { id: 456 }, { id: 19786 }];
        const reviewHistoryEntryCollection: IReviewHistoryEntry[] = [{ id: 123 }];
        expectedResult = service.addReviewHistoryEntryToCollectionIfMissing(reviewHistoryEntryCollection, ...reviewHistoryEntryArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const reviewHistoryEntry: IReviewHistoryEntry = { id: 123 };
        const reviewHistoryEntry2: IReviewHistoryEntry = { id: 456 };
        expectedResult = service.addReviewHistoryEntryToCollectionIfMissing([], reviewHistoryEntry, reviewHistoryEntry2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(reviewHistoryEntry);
        expect(expectedResult).toContain(reviewHistoryEntry2);
      });

      it('should accept null and undefined values', () => {
        const reviewHistoryEntry: IReviewHistoryEntry = { id: 123 };
        expectedResult = service.addReviewHistoryEntryToCollectionIfMissing([], null, reviewHistoryEntry, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(reviewHistoryEntry);
      });

      it('should return initial array if no ReviewHistoryEntry is added', () => {
        const reviewHistoryEntryCollection: IReviewHistoryEntry[] = [{ id: 123 }];
        expectedResult = service.addReviewHistoryEntryToCollectionIfMissing(reviewHistoryEntryCollection, undefined, null);
        expect(expectedResult).toEqual(reviewHistoryEntryCollection);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
