import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { of, Subject, from } from 'rxjs';

import { UserWatchListService } from '../service/user-watch-list.service';
import { IUserWatchList, UserWatchList } from '../user-watch-list.model';

import { IUser } from 'app/entities/user/user.model';
import { UserService } from 'app/entities/user/user.service';

import { UserWatchListUpdateComponent } from './user-watch-list-update.component';

describe('UserWatchList Management Update Component', () => {
  let comp: UserWatchListUpdateComponent;
  let fixture: ComponentFixture<UserWatchListUpdateComponent>;
  let activatedRoute: ActivatedRoute;
  let userWatchListService: UserWatchListService;
  let userService: UserService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, RouterTestingModule.withRoutes([])],
      declarations: [UserWatchListUpdateComponent],
      providers: [
        FormBuilder,
        {
          provide: ActivatedRoute,
          useValue: {
            params: from([{}]),
          },
        },
      ],
    })
      .overrideTemplate(UserWatchListUpdateComponent, '')
      .compileComponents();

    fixture = TestBed.createComponent(UserWatchListUpdateComponent);
    activatedRoute = TestBed.inject(ActivatedRoute);
    userWatchListService = TestBed.inject(UserWatchListService);
    userService = TestBed.inject(UserService);

    comp = fixture.componentInstance;
  });

  describe('ngOnInit', () => {
    it('Should call User query and add missing value', () => {
      const userWatchList: IUserWatchList = { id: 456 };
      const user: IUser = { id: 25556 };
      userWatchList.user = user;

      const userCollection: IUser[] = [{ id: 31718 }];
      jest.spyOn(userService, 'query').mockReturnValue(of(new HttpResponse({ body: userCollection })));
      const additionalUsers = [user];
      const expectedCollection: IUser[] = [...additionalUsers, ...userCollection];
      jest.spyOn(userService, 'addUserToCollectionIfMissing').mockReturnValue(expectedCollection);

      activatedRoute.data = of({ userWatchList });
      comp.ngOnInit();

      expect(userService.query).toHaveBeenCalled();
      expect(userService.addUserToCollectionIfMissing).toHaveBeenCalledWith(userCollection, ...additionalUsers);
      expect(comp.usersSharedCollection).toEqual(expectedCollection);
    });

    it('Should update editForm', () => {
      const userWatchList: IUserWatchList = { id: 456 };
      const user: IUser = { id: 60168 };
      userWatchList.user = user;

      activatedRoute.data = of({ userWatchList });
      comp.ngOnInit();

      expect(comp.editForm.value).toEqual(expect.objectContaining(userWatchList));
      expect(comp.usersSharedCollection).toContain(user);
    });
  });

  describe('save', () => {
    it('Should call update service on save for existing entity', () => {
      // GIVEN
      const saveSubject = new Subject<HttpResponse<UserWatchList>>();
      const userWatchList = { id: 123 };
      jest.spyOn(userWatchListService, 'update').mockReturnValue(saveSubject);
      jest.spyOn(comp, 'previousState');
      activatedRoute.data = of({ userWatchList });
      comp.ngOnInit();

      // WHEN
      comp.save();
      expect(comp.isSaving).toEqual(true);
      saveSubject.next(new HttpResponse({ body: userWatchList }));
      saveSubject.complete();

      // THEN
      expect(comp.previousState).toHaveBeenCalled();
      expect(userWatchListService.update).toHaveBeenCalledWith(userWatchList);
      expect(comp.isSaving).toEqual(false);
    });

    it('Should call create service on save for new entity', () => {
      // GIVEN
      const saveSubject = new Subject<HttpResponse<UserWatchList>>();
      const userWatchList = new UserWatchList();
      jest.spyOn(userWatchListService, 'create').mockReturnValue(saveSubject);
      jest.spyOn(comp, 'previousState');
      activatedRoute.data = of({ userWatchList });
      comp.ngOnInit();

      // WHEN
      comp.save();
      expect(comp.isSaving).toEqual(true);
      saveSubject.next(new HttpResponse({ body: userWatchList }));
      saveSubject.complete();

      // THEN
      expect(userWatchListService.create).toHaveBeenCalledWith(userWatchList);
      expect(comp.isSaving).toEqual(false);
      expect(comp.previousState).toHaveBeenCalled();
    });

    it('Should set isSaving to false on error', () => {
      // GIVEN
      const saveSubject = new Subject<HttpResponse<UserWatchList>>();
      const userWatchList = { id: 123 };
      jest.spyOn(userWatchListService, 'update').mockReturnValue(saveSubject);
      jest.spyOn(comp, 'previousState');
      activatedRoute.data = of({ userWatchList });
      comp.ngOnInit();

      // WHEN
      comp.save();
      expect(comp.isSaving).toEqual(true);
      saveSubject.error('This is an error!');

      // THEN
      expect(userWatchListService.update).toHaveBeenCalledWith(userWatchList);
      expect(comp.isSaving).toEqual(false);
      expect(comp.previousState).not.toHaveBeenCalled();
    });
  });

  describe('Tracking relationships identifiers', () => {
    describe('trackUserById', () => {
      it('Should return tracked User primary key', () => {
        const entity = { id: 123 };
        const trackResult = comp.trackUserById(0, entity);
        expect(trackResult).toEqual(entity.id);
      });
    });
  });
});
